//
//  WTSmaleScale3DTrackingViewController.m
//  Native Examples
//
//  Created by Andreas Schacherbauer on 01/10/15.
//  Copyright © 2015 Wikitude. All rights reserved.
//

#import "WT3DTrackingViewController.h"

#import <WikitudeNativeSDK/WikitudeNativeSDK.h>

#import "WikitudeLicense.h"
#import "StrokedRectangle.h"
#import "ExternalRenderer.h"


@interface WT3DTrackingViewController () <WTWikitudeNativeSDKDelegate, WTTrackingMapRecorderDelegate, WTClientTrackerDelegate>

@property (nonatomic, strong) WTWikitudeNativeSDK                           *wikitudeSDK;
@property (nonatomic, strong) WTClientTracker                               *clientTracker;

@property (nonatomic, assign) WTTrackingMapRecordingQuality                 currentTrackingMapRecordingQuality;

@property (nonatomic, strong) EAGLContext                                   *sharedWikitudeEAGLCameraContext;

@property (nonatomic, copy) WTWikitudeUpdateHandler                         wikitudeUpdateHandler;
@property (nonatomic, copy) WTWikitudeDrawHandler                           wikitudeDrawHandler;

@property (nonatomic, assign) BOOL                                          isTracking;

@property (nonatomic, strong) ExternalRenderer                              *renderer;
@property (nonatomic, strong) StrokedRectangle                              *renderableRectangle;

@end

@implementation WT3DTrackingViewController
#pragma mark UIViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    self.renderer = [[ExternalRenderer alloc] init];
    self.renderableRectangle = [[StrokedRectangle alloc] init];

    self.wikitudeSDK = [[WTWikitudeNativeSDK alloc] initWithRenderingMode:WTRenderingMode_External delegate:self];
    [self.wikitudeSDK setLicenseKey:kWTLicenseKey];

    [[self.wikitudeSDK trackingMapRecorder] setDelegate:self];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didReceiveApplicationWillResignActiveNotification:) name:UIApplicationWillResignActiveNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didReceiveApplicationDidBecomeActiveNotification:) name:UIApplicationDidBecomeActiveNotification object:nil];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    
    [self.renderer setupRenderingWithLayer:[self.eaglView eaglLayer]];
    [self.renderer startRenderLoopWithRenderBlock:[self renderBlock]];
    
    [self.wikitudeSDK start:nil completion:^(BOOL isRunning, NSError * __nonnull error) {
        if ( !isRunning )
        {
            NSLog(@"Wikitude SDK is not running. Reason: %@", [error localizedDescription]);
        }
    }];

    [self showExampleIntroduction];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];

    [self setNavigationItemPrompt:nil withNavigationBarTintColor:nil];    
}

- (void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:animated];
    
    [self.wikitudeSDK stop];
    
    [self.renderer stopRenderLoop];
    [self.renderer teardownRendering];
    
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(id<UIViewControllerTransitionCoordinator>)coordinator
{
    [super viewWillTransitionToSize:size withTransitionCoordinator:coordinator];
    
    [self.wikitudeSDK shouldTransitionToSize:size withTransitionCoordinator:coordinator];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - ExternalRenderer render loop
- (ExternalRenderBlock)renderBlock
{
    return ^ (CADisplayLink *displayLink) {
        if ( self.wikitudeUpdateHandler
            &&
            self.wikitudeDrawHandler )
        {
            self.wikitudeUpdateHandler();
            self.wikitudeDrawHandler();
        }
        
        [self.renderer bindBuffer];
        
        if ( _isTracking )
        {
            [self.renderableRectangle drawInContext:[self.renderer internalContext]];
        }
    };
}

#pragma mark - Private Methods
- (void)stopTrackingMapRecording:(id)sender
{
    if ( WTTrackingMapRecordingQuality_Good == _currentTrackingMapRecordingQuality )
    {
        [self stopAndLoadTrackingMapRecordingInternal];
    }
    else
    {
        __weak typeof(self) weakSelf = self;
        UIAlertController *insufficientTrackingQualityAlertController = [UIAlertController alertControllerWithTitle:@"Tracking Map quality not sufficient." message:@"In order to experience a well working 3d tracking example, please continue recording until the qualiy indicator says 'Good'." preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *continueAnywayAction = [UIAlertAction actionWithTitle:@"Continue Anyways" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
            [weakSelf stopAndLoadTrackingMapRecordingInternal];
        }];

        UIAlertAction *restartRecordingAction = [UIAlertAction actionWithTitle:@"Restart Recording" style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
            [[weakSelf.wikitudeSDK trackingMapRecorder] cancelTrackingMapRecording];
            [weakSelf startTrackingMapRecordingInternal];
        }];
        [insufficientTrackingQualityAlertController addAction:continueAnywayAction];
        [insufficientTrackingQualityAlertController addAction:restartRecordingAction];


        [self presentViewController:insufficientTrackingQualityAlertController animated:YES completion:nil];
    }
}

/* Called from UIAlertActions */
- (void)startTrackingMapRecordingInternal
{
    [self setNavigationItemPrompt:nil withNavigationBarTintColor:nil];

    if ( ![[self.wikitudeSDK trackingMapRecorder] isRecording] )
    {
        __weak typeof(self) weakSelf = self;
        [[self.wikitudeSDK trackingMapRecorder] startTrackingMapRecording:^(BOOL isRecording, NSError * _Nullable error) {
            if ( !isRecording )
            {
                NSLog(@"Unable to start Tracking Map recording. Reason: %@", [error localizedDescription]);
            }
            else
            {
                if ( nil == weakSelf.navigationItem.rightBarButtonItem )
                {
                    [weakSelf.navigationItem setRightBarButtonItem:[[UIBarButtonItem alloc] initWithTitle:@"Stop Recording" style:UIBarButtonItemStylePlain target:weakSelf action:@selector(stopTrackingMapRecording:)] animated:YES];
                }
            }
        }];
    }
}

- (void)stopAndLoadTrackingMapRecordingInternal
{
    if ( [[self.wikitudeSDK trackingMapRecorder] isRecording] )
    {
        [[self.navigationItem rightBarButtonItem] setEnabled:NO];
        [self setNavigationItemPrompt:@"Saving Tracking Map..." withNavigationBarTintColor:nil];

        /* For this example, the Tracking Map is always called the same and each Tracking Map will override the already existing one. */
        __weak typeof(self) weakSelf = self;
        [[self.wikitudeSDK trackingMapRecorder] stopTrackingMapRecording:@"3d_tracking_map" completion:^(NSURL * _Nullable trackingMapURL, NSError * _Nullable error) {

            [weakSelf setNavigationItemPrompt:nil withNavigationBarTintColor:nil];
            if ( nil == trackingMapURL )
            {
                NSLog(@"Unable to save recorded Tracking Map. Reason: %@", [error localizedDescription]);
            }
            else
            {
                [weakSelf.navigationItem setRightBarButtonItem:nil animated:YES];
                weakSelf.clientTracker = [[weakSelf.wikitudeSDK trackerManager] create3DClientTrackerFromURL:trackingMapURL andDelegate:weakSelf];
            }
        }];
    }
}

- (void)showExampleIntroduction
{
    __weak typeof(self) weakSelf = self;
    UIAlertController *exampleIntroductionAlertController = [UIAlertController alertControllerWithTitle:@"Tracking Map Required" message:@"For this example to work, a Tracking Map needs to be recorded. \nPlease tap 'Start' below to start recording a Tracking Map.\nNote: Point the device to the center of the scene that you're about to record." preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *startMapRecordingAction = [UIAlertAction actionWithTitle:@"Start Recording" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [weakSelf startTrackingMapRecordingInternal];
    }];
    [exampleIntroductionAlertController addAction:startMapRecordingAction];

    [self presentViewController:exampleIntroductionAlertController animated:YES completion:nil];
}

- (void)setNavigationItemPrompt:(NSString *)prompt withNavigationBarTintColor:(UIColor *)tintColor
{
    if ( nil == tintColor )
    {
        tintColor = [UIColor whiteColor];
    }

    [self.navigationController.navigationBar setBarTintColor:tintColor];
    [self.navigationItem setPrompt:prompt];
}
#pragma mark - Notifications
- (void)didReceiveApplicationWillResignActiveNotification:(NSNotification *)notification
{
    [self.wikitudeSDK stop];

    [self.renderer stopRenderLoop];
}

- (void)didReceiveApplicationDidBecomeActiveNotification:(NSNotification *)notification
{
    [self.renderer startRenderLoopWithRenderBlock:[self renderBlock]];

    [self.wikitudeSDK start:nil completion:^(BOOL isRunning, NSError * __nonnull error) {
        if ( !isRunning ) {
            NSLog(@"Wikitude SDK is not running. Reason: %@", [error localizedDescription]);
        }
    }];
}

#pragma mark - Delegation
#pragma mark WTWikitudeNativeSDKDelegte
- (void)wikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK didCreatedExternalUpdateHandler:(WTWikitudeUpdateHandler __nonnull)updateHandler
{
    self.wikitudeUpdateHandler = updateHandler;
}

- (void)wikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK didCreatedExternalDrawHandler:(WTWikitudeDrawHandler __nonnull)drawHandler
{
    self.wikitudeDrawHandler = drawHandler;
}

- (EAGLContext *)eaglContextForVideoCameraInWikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK
{
    if (!_sharedWikitudeEAGLCameraContext )
    {
        EAGLContext *rendererContext = [self.renderer internalContext];
        self.sharedWikitudeEAGLCameraContext = [[EAGLContext alloc] initWithAPI:[rendererContext API] sharegroup:[rendererContext sharegroup]];
    }
    return self.sharedWikitudeEAGLCameraContext;
}

- (CGRect)eaglViewSizeForExternalRenderingInWikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK
{
    return self.eaglView.bounds;
}

- (void)wikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK didEncounterInternalError:(NSError * __nonnull)error
{
    NSLog(@"Internal Wikitude SDK error encounterd. %@", [error localizedDescription]);
}

#pragma mark WTClientTrackerDelegate

- (void)baseTracker:(nonnull WTBaseTracker *)baseTracker didRecognizedTarget:(nonnull WTImageTarget *)recognizedTarget
{
    NSLog(@"recognized target '%@'", [recognizedTarget name]);
    _isTracking = YES;
}

- (void)baseTracker:(nonnull WTBaseTracker *)baseTracker didTrackTarget:(nonnull WTImageTarget *)trackedTarget
{
    [self.renderableRectangle setProjectionMatrix:trackedTarget.projection];
    [self.renderableRectangle setModelViewMatrix:trackedTarget.modelView];
}

- (void)baseTracker:(nonnull WTBaseTracker *)baseTracker didLostTarget:(nonnull WTImageTarget *)lostTarget
{
    NSLog(@"lost target '%@'", [lostTarget name]);
    _isTracking = NO;
}


- (void)clientTracker:(nonnull WTClientTracker *)clientTracker didFinishedLoadingTargetCollectionFromURL:(nonnull NSURL *)URL
{
    NSLog(@"Client tracker loaded");
}

- (void)clientTracker:(nonnull WTClientTracker *)clientTracker didFailToLoadTargetCollectionFromURL:(nonnull NSURL *)URL withError:(nonnull NSError *)error
{
    NSLog(@"Unable to load client tracker from URL '%@'. Reason: %@", [URL absoluteString], [error localizedDescription]);
}

#pragma mark WTTrackerManagerDelegate
- (void)trackingMapRecorder:(WTTrackingMapRecorder *)trackingMapRecorder didChangeTrackingMapRecordingQualityFrom:(WTTrackingMapRecordingQuality)oldTrackingQuality toQuality:(WTTrackingMapRecordingQuality)newTrackingQuality
{
    UIColor *trackingMapRecordingQualityBarTintColor = nil;
    NSString *trackingMapRecordingQualityText = nil;

    _currentTrackingMapRecordingQuality = newTrackingQuality;

    if ( WTTrackingMapRecordingQuality_Bad == newTrackingQuality )
    {
        trackingMapRecordingQualityBarTintColor = [UIColor colorWithRed:1 green:0.204 blue:0.125 alpha:1];
        trackingMapRecordingQualityText = @"Bad";
    }
    else if ( WTTrackingMapRecordingQuality_Average == newTrackingQuality )
    {
        trackingMapRecordingQualityBarTintColor = [UIColor colorWithRed:1 green:0.851 blue:0 alpha:1];
        trackingMapRecordingQualityText = @"Average";
    }
    else
    {
        trackingMapRecordingQualityBarTintColor = [UIColor colorWithRed:0.420 green:1 blue:0 alpha:1];
        trackingMapRecordingQualityText = @"Good";
    }

    if ( trackingMapRecordingQualityBarTintColor
        &&
        trackingMapRecordingQualityText )
    {
        [self setNavigationItemPrompt:[NSString stringWithFormat:@"Recorded Tracking Map Quality: %@", trackingMapRecordingQualityText] withNavigationBarTintColor:trackingMapRecordingQualityBarTintColor];
    }
}

@end
